/*  Copyright (c) 2012, Intencity Cloud Technologies LLC. See website for details.*/

if (typeof log == "undefined") {
	var log = function(msg) {
		if (typeof console != "undefined") {
			console.log(msg);
		}
	};
}


network = new function() {

	this.movieName = "flash-network";
	this.onstatus = null;
	this.onnetworkchange = null;
	this.interfaces = [];
	
	// the API's object list
	this._lastObjId = 0;
	this._lastMsgId = 0;
	this._objects = {};
	this._pending = {};
	this._receivedPending = "";
	
	this._debug = false;
	
	this.log = function(msg) {
		if (this._debug)
			log(msg);
	};
	
	this.movie = function() {
	    var isIE = navigator.appName.indexOf("Microsoft") != -1;
	    return (isIE) ? window[this.movieName] : document[this.movieName];
	};
	
	this.onPropertyChange = function(event) {
		network.log(event.objectID + " " + event.property + " " + event.oldValue + "=>" + event.newValue);
		if (event.objectID == this.movieName && event.property == "status") {
			if (this.onstatus != null) {
				this.onstatus(event.newValue);
			}
		}
	};
	
	this.onReceiveData = function(event) {
		network.log(event.objectID + " data " + event.data);
		if (event.data) {
			this._receivedPending += event.data;
			if (this._receivedPending && this._receivedPending.charAt(this._receivedPending.length-1) == "\n") {
				var parts = this._receivedPending.split("\n");
				this._receivedPending = "";
				for (var i=0; i<parts.length; ++i) {
					var part = parts[i];
					part = part.replace(/^\s+|\s+$/g, "");
					if (part) {
						try {
							var obj = JSON.parse(part);
							this._received(obj);
						}
						catch (ex) {
							network.log(ex['stack'] !== undefined ? ex.stack : ex);
						}
					}
				}
			}
		}
	};
	
	this._received = function(obj) {
		if (obj.msgid != undefined && obj.result != undefined) {
			var msgid = obj.msgid;
			if (this._pending[msgid] != undefined) {
				var callback = this._pending[msgid];
				if (obj.complete != undefined && obj.complete) {
					// delete the callback
					delete this._pending[msgid];
				}
				
				if (callback != null) {
					if (obj.result == "success") {
						callback(obj.args != undefined ? obj.args : []);
					} 
					else {
						network.log("received failed result");
					}
				}
			}
		}
		else if (obj.event != undefined) {
			var event = obj.event;
			var id = obj.id;
			
			if (this._objects[id] != undefined) {
				var obj1 = this._objects[id];
				for (var s in event) {
					if (s != "type" && typeof event[s] == "string") {
						if (event[s].substr(0, 4) == "b64:") {
							event[s] = Base64.decode(event[s].substr(4));
						}
						else if (event[s].substr(0, 4) == "oid:") {
							var oid = event[s].substr(4);
							if (this._objects[oid] != undefined) {
								event[s] = this._objects[oid];
							}
							else {
								network.log("the object " + event[s] + " does not exist");
								event[s] = null;
							}
						}
					}
				}
				if (event.type == "propertyChange") {
					obj1.onPropertyChange.apply(obj1, [event]);
				}
				obj1.dispatchEvent.apply(obj1, [event]);
			}
			else if (!id && event.property == "interfaces") {
				this.interfaces = event.newValue;
				if (this.onnetworkchange != null) {
					this.onnetworkchange(event.newValue);
				}
			}
		}
		else if (obj.method != undefined) {
			if (obj.method == "new") {
				network.log("created on server: " + obj.type + " " + obj.id);
				var actual = null;
				if (obj.type == "flash.net.Socket")
					actual = new network.Socket(true);
				else if (obj.type == "flash.net.ServerSocket")
					actual = new network.ServerSocket(true);
				else if (obj.type == "flash.net.DatagramSocket")
					actual = new network.DatagramSocket(true);
				else
					network.log("type not found " + obj.type);
				if (actual != null) {
					actual._id = obj.id;
					this._objects[obj.id] = actual;
				}
			}
		}
	};
	
	this._send = function(obj) {
		var msgid = "" + (++this._lastMsgId);
		obj["msgid"] = msgid;
		var movie = this.movie();
		var data = JSON.stringify(obj);
		network.log("sending " + data);
		movie.send(data + "\n");
		return msgid;
	}
	
	this._create = function(obj, type, args, callback) {
		var id = null;
		while (id == null || this._objects[id] != undefined) {
			++this._lastObjId;
			id = "" + this._lastObjId;
		}
		this._objects[id] = obj;
		var msgid = this._send({"method": "new", "id": id, "type": type, "args": args});
		this._pending[msgid] = (callback != undefined ? callback : null);
		return id;
	};
	
	this._destroy = function(id, callback) {
		if (this._objects[id] != undefined) {
			delete this._objects[id];
			var msgid = this._send({"method": "delete", "id": id});
			this._pending[msgid] = (callback != undefined ? callback : null);
		}
	};
	
	this._call = function(method, args, callback) {
		var msgid = this._send({"method": method, "args": args});
		this._pending[msgid] = (callback != undefined ? callback : null);
	};
	
	this._methodcall = function(id, method, args, callback) {
		if (this._objects[id] != undefined) {
			var msgid = this._send({"method": method, "id": id, "args": args});
			this._pending[msgid] = (callback != undefined ? callback : null);
		}
	};
	
	this._eventadd = function(id, eventType) {
		if (this._objects[id] != undefined) {
			var msgid = this._send({"method": "addEventListener", "id": id, "args": [eventType]});
			this._pending[msgid] = null;
		}
	};
	
	this._eventremove = function(id, eventType) {
		if (this._objects[id] != undefined) {
			var msgid = this._send({"method": "removeEventListener", "id": id, "args": [eventType]});
			this._pending[msgid] = null;
		}
	};
	
};



// EventDispatcher

(function(network) {

	function EventDispatcher() {
		this._eventListeners = {};
		this.addEventListener = function(eventType, handler) {
			var add = false;
			if (this._eventListeners[eventType] === undefined) { 
				this._eventListeners[eventType] = [];
				add = true;
			}
				
			this._eventListeners[eventType].push(handler);
			if (add) {
				network._eventadd(this._id, eventType);
			}
		};
		
		this.removeEventListener = function(eventType, handler) {
			var remove = false;
			if (handler != undefined && this._eventListeners[eventType] !== undefined) {
				var index = this._eventListeners[eventType].indexOf(handler);
				this._eventListeners[eventType].splice(index, 1);
				if (this._eventListeners[eventType].length == 0) { 
					delete this._eventListeners[eventType];
					remove = true;
				}
			} else if (this._eventListeners[eventType] !== undefined) {
				delete this._eventListeners[eventType];
				remove = true;
			}
			if (remove) {
				network._eventremove(this._id, eventType);
			}
		};
		
		this.dispatchEvent = function(event) {
			if (this._eventListeners[event.type] != undefined) {
				var listeners = this._eventListeners[event.type];
				for (var i=0; i<listeners.length; ++i) {
					var func = listeners[i];
					try {
						func(event);
					} catch (e) {
						network.log("exception: " + e);
					}
				}
			}
		};
		
		this.onPropertyChange = function(event) {
			if (typeof this[event.property] != undefined) {
				network.log("property-change id: " + this._id + " property: " + event.property + " value: " + event.newValue);
				this[event.property] = event.newValue;
			}
			else {
				network.log("ignored property change " + event.property + " " + event.newValue);
			}
		};
	}
	
	network.EventDispatcher = EventDispatcher;
})(network);



// Socket

(function(network) {

	function Socket(created) {
		this.base = network.EventDispatcher;
		this.base();
		delete this.base;
		
		this.connected = false;
		this.timeout = 20;
		this.localAddress = null;
		this.localPort = NaN;
		this.remoteAddress = null;
		this.remotePort = NaN;
		
		if (!created)
			this._id = network._create(this, "flash.net.Socket", []);
	}
	
	Socket.prototype.connect = function(host, port) {
		//if (this.remoteAddress != host)
		//	this.onPropertyChange({property: "remoteAddress", "oldValue": this.remoteAddress, "newValue": host});
		//if (this.remotePort != port)
		//	this.onPropertyChange({property: "remotePort", "oldValue": this.remotePort, "newValue": port});
		
		network._methodcall(this._id, "connect", [host, port]);
	};
	
	Socket.prototype.close = function() {
		//if (this.remoteAddress != null)
		//	this.onPropertyChange({property: "remoteAddress", "oldValue": this.remoteAddress, "newValue": null});
		//if (this.remotePort != null)
		//	this.onPropertyChange({property: "remotePort", "oldValue": this.remotePort, "newValue": null});
		
		network._methodcall(this._id, "close", []);
	};
	
	Socket.prototype.writeBytes = function(data) {
		data = "b64:" + Base64.encode(data).replace(/\n/g, "");
		network._methodcall(this._id, "writeBytes", [data]);
		network._methodcall(this._id, "flush", []); // on windows flush is required to send
	};
	
	network.Socket = Socket;
})(network);


(function(network) {

	function ServerSocket(created) {
		this.base = network.EventDispatcher;
		this.base();
		delete this.base;
		
		this.bound = false;
		this.listening = false;
		this.localAddress = null;
		this.localPort = NaN;
		
		if (!created)
			this._id = network._create(this, "flash.net.ServerSocket", []);
	}
	
	ServerSocket.prototype.bind = function(localPort, localAddress) {
		localPort = (localPort != undefined ? localPort : 0);
		localAddress = (localAddress != undefined ? localAddress : "0.0.0.0");
		
		//if (this.localAddress != null)
		//	this.onPropertyChange({property: "localAddress", "oldValue": this.localAddress, "newValue": localAddress});
		//if (this.localPort != null)
		//	this.onPropertyChange({property: "localPort", "oldValue": this.localPort, "newValue": localPort});
		
		network._methodcall(this._id, "bind", [localPort, localAddress]);
	};
	
	ServerSocket.prototype.close = function() {
		//if (this.localAddress != null)
		//	this.onPropertyChange({property: "localAddress", "oldValue": this.localAddress, "newValue": null});
		//if (this.localPort != null)
		//	this.onPropertyChange({property: "localPort", "oldValue": this.localPort, "newValue": null});
		//if (this.listening != false)
		//	this.onPropertyChange({property: "listening", "oldValue": this.listening, "newValue": false});
		
		network._methodcall(this._id, "close", []);
	};
	
	ServerSocket.prototype.listen = function(backlog) {
		//if (this.listening != true)
		//	this.onPropertyChange({property: "listening", "oldValue": this.listening, "newValue": true});
		
		network._methodcall(this._id, "listen", [backlog != undefined ? backlog : 0]);
	};
	
	network.ServerSocket = ServerSocket;
})(network);


(function(network) {

	function DatagramSocket(created) {
		this.base = network.EventDispatcher;
		this.base();
		delete this.base;
		
		this.bound = false;
		this.connected = false;
		this.localAddress = null;
		this.localPort = NaN;
		this.remoteAddress = null;
		this.remotePort = NaN;
		
		if (!created)
			this._id = network._create(this, "flash.net.DatagramSocket", []);
	}
	
	DatagramSocket.prototype.bind = function(localPort, localAddress) {
		localPort = (localPort != undefined ? localPort : 0);
		localAddress = (localAddress != undefined ? localAddress : "0.0.0.0");
		
		//if (this.localAddress != localAddress) 
		//	this.onPropertyChange({property: "localAddress", "oldValue": this.localAddress, "newValue": localAddress});
		//if (this.localPort != localPort)
		//	this.onPropertyChange({property: "localPort", "oldValue": this.localPort, "newValue": localPort});
		
		network._methodcall(this._id, "bind", [localPort, localAddress]);
	};
	
	DatagramSocket.prototype.close = function() {
		//if (this.localAddress != null)
		//	this.onPropertyChange({property: "localAddress", "oldValue": this.localAddress, "newValue": null});
		//if (this.localPort != null)
		//	this.onPropertyChange({property: "localPort", "oldValue": this.localPort, "newValue": null});
		//if (this.connected != false) 
		//	this.onPropertyChange({property: "connected", "oldValue": this.connected, "newValue": false});
		
		network._methodcall(this._id, "close", []);
	};
	
	DatagramSocket.prototype.connect = function(remoteAddress, remotePort) {
		//if (this.connected != true)
		//	this.onPropertyChange({property: "connected", "oldValue": this.connected, "newValue": true});
		
		network._methodcall(this._id, "connect", [remoteAddress, remotePort]);
	};
	
	DatagramSocket.prototype.receive = function() {
		network._methodcall(this._id, "receive", []);
	};
	
	DatagramSocket.prototype.send = function(data, address, port) {
		data = "b64:" + Base64.encode(data).replace(/\n/g, "");
		network._methodcall(this._id, "send", [data, 0, 0, address != undefined ? address : null, port != undefined ? port : 0]);
	};
	
	network.DatagramSocket = DatagramSocket;
})(network);


(function(network) {

	function RealTimeSocket(created) {
		this.base = network.EventDispatcher;
		this.base();
		delete this.base;
		
		this.bound = false;
		this.connected = false;
		this.localAddress = null;
		this.localPort = NaN;
		this.remoteAddress = null;
		this.remotePort = NaN;
		this.localControlAddress = null;
		this.localControlPort = NaN;
		this.remoteControlAddress = null;
		this.remoteControlPort = NaN;
		
		if (!created)
			this._id = network._create(this, "com.theintencity.RealTimeSocket", []);
	}
	
	RealTimeSocket.prototype.bind = function(localPort, localAddress, localControlPort, localControlAddress) {
		localPort = (localPort != undefined ? localPort : 0);
		localAddress = (localAddress != undefined ? localAddress : "0.0.0.0");
		localControlPort = (localControlPort != undefined ? localControlPort : 0);
		localControlAddress = (localControlAddress != undefined ? localControlAddress : "0.0.0.0");
		
		network._methodcall(this._id, "bind", [localPort, localAddress, localControlPort, localControlAddress]);
	};
	
	RealTimeSocket.prototype.close = function() {
		network._methodcall(this._id, "close", []);
	};
	
	RealTimeSocket.prototype.connect = function(remoteAddress, remotePort, remoteControlAddress, remoteControlPort) {
		remoteControlAddress = (remoteControlAddress != undefined ? remoteControlAddress : null);
		remoteControlPort = (remoteControlPort != undefined ? remoteControlPort : 0);
		network._methodcall(this._id, "connect", [remoteAddress, remotePort, remoteControlAddress, remoteControlPort]);
	};
	
	// whether to receive non media data in the "data" event?
	RealTimeSocket.prototype.receive = function(pt) {
		network._methodcall(this._id, "receive", [pt]);
	};
	
	// send non-media data on the socket. It must be a connected socket.
	RealTimeSocket.prototype.send = function(data, pt, marker, ts) {
		ts = (ts != undefined ? ts : 0);
		marker = (marker != undefined ? marker : false);
		
		data = "b64:" + Base64.encode(data).replace(/\n/g, "");
		network._methodcall(this._id, "send", [data, pt, marker, ts]);
	};
	
	network.RealTimeSocket = RealTimeSocket;
})(network);


(function(network) {

	function RealTimeGateway(created) {
		this.base = network.EventDispatcher;
		this.base();
		delete this.base;
		
		this.playing = false;
		this.publishing = false;
		this.playurl = null;
		this.publishurl = null;
		
		if (!created)
			this._id = network._create(this, "com.theintencity.RealTimeGateway", []);
	}
	
	RealTimeGateway.prototype.setAudio = function(socket, receiveTypes, sendTypes) {
		network._methodcall(this._id, "setAudio", ["oid:" + socket._id, receiveTypes, sendTypes]);
	};
	
	RealTimeGateway.prototype.setVideo = function(socket, receiveTypes, sendTypes) {
		network._methodcall(this._id, "setVideo", ["oid:" + socket._id, receiveTypes, sendTypes]);
	};
	
	RealTimeGateway.prototype.close = function() {
		network._methodcall(this._id, "close", []);
	};
	
	network.RealTimeGateway = RealTimeGateway;
})(network);




(function(network) {

	function DNSResolver(created) {
		this.base = network.EventDispatcher;
		this.base();
		delete this.base;
		
		if (!created)
			this._id = network._create(this, "flash.net.dns.DNSResolver", []);
	}
	
	DNSResolver.prototype.resolve = function(host, type) {
		network._methodcall(this._id, "resolve", [host, type]);
	};
	
	network.DNSResolver = DNSResolver;
})(network);
